[data-md-color-scheme="default"] {
  --custom-background: white;
  --param-color: rgb(119, 82, 3);
}

[data-md-color-scheme="slate"] {
  --custom-background: rgb(190, 190, 190);
  --param-color: rgb(207, 180, 120);
}

.class {
  color: rgb(8, 34, 8);
  background-color: rgba(139, 226, 200, 0.5);
  padding: 0.25rem;
  margin-right: 0.5rem;
  font-family: "Roboto Mono", monospace;
  font-size: small;
}
.class::before {
  content: "class";
}
.attr {
  color: rgb(32, 34, 8);
  background-color: rgb(226, 181, 139, 0.5);
  padding: 0.25rem;
  margin-right: 0.5rem;
  font-family: "Roboto Mono", monospace;
  font-size: small;
}
.attr::before {
  content: "attr";
}
.meth {
  color: rgb(8, 17, 34);
  background-color: rgb(170, 168, 230, 0.5);
  padding: 0.25rem;
  margin-right: 0.5rem;
  font-family: "Roboto Mono", monospace;
  font-size: small;
}
.meth::before {
  content: "meth";
}
.func {
  color: rgb(8, 17, 34);
  background-color: rgb(170, 168, 230, 0.5);
  padding: 0.25rem;
  margin-right: 0.5rem;
  font-family: "Roboto Mono", monospace;
  font-size: small;
}
.func::before {
  content: "func";
}
.prop {
  color: rgb(8, 17, 34);
  background-color: rgba(230, 168, 199, 0.5);
  padding: 0.25rem;
  margin-right: 0.5rem;
  font-family: "Roboto Mono", monospace;
  font-size: small;
}
.prop::before {
  content: "prop";
}

.param,
.returns {
  color: var(--param-color);
  padding: 0.25rem;
  margin-right: 0.5rem;
  font-family: "Roboto Mono", monospace;
}

.light-dark-bg > img {
  background-color: var(--custom-background);
  border: 1px solid gray;
}

.img-light-dark-bg {
  background-color: var(--custom-background);
  border: 1px solid gray;
  display: block;
  margin-left: auto;
  margin-right: auto;
}
